@@ -1,5 +1,6 @@
 from __future__ import print_function, division, absolute_import
 
+from distutils.version import LooseVersion
 from functools import partial
 import logging
 import math
@@ -8,8 +9,7 @@ from numbers import Number
 from operator import add
 import os
 
-from bokeh.application import Application
-from bokeh.application.handlers.function import FunctionHandler
+import bokeh
 from bokeh.layouts import column, row
 from bokeh.models import (ColumnDataSource, DataRange1d, HoverTool, ResetTool,
                           PanTool, WheelZoomTool, TapTool, OpenURL, Range1d, Plot, Quad,
@@ -18,6 +18,7 @@ from bokeh.models import (ColumnDataSource, DataRange1d, HoverTool, ResetTool,
 from bokeh.models.widgets import DataTable, TableColumn
 from bokeh.plotting import figure
 from bokeh.palettes import Viridis11
+from bokeh.themes import Theme
 from bokeh.transform import factor_cmap
 from bokeh.io import curdoc
 from toolz import pipe, merge
@@ -28,7 +29,7 @@ except ImportError:
     np = False
 
 from . import components
-from .components import DashboardComponent, ProfileTimePlot
+from .components import (DashboardComponent, ProfileTimePlot, ProfileServer)
 from .core import BokehServer
 from .worker import SystemMonitor, counters_doc
 from .utils import transpose
@@ -37,7 +38,7 @@ from ..utils import log_errors, format_bytes, format_time
 from ..diagnostics.progress_stream import color_of, progress_quads, nbytes_bar
 from ..diagnostics.progress import AllProgress
 from ..diagnostics.graph_layout import GraphLayout
-from .task_stream import TaskStreamPlugin
+from ..diagnostics.task_stream import TaskStreamPlugin
 
 try:
     from cytoolz.curried import map, concat, groupby, valmap, first
@@ -49,15 +50,12 @@ logger = logging.getLogger(__name__)
 
 PROFILING = False
 
-import jinja2
-
-with open(os.path.join(os.path.dirname(__file__), 'template.html')) as f:
-    template_source = f.read()
-
-template = jinja2.Template(template_source)
+from jinja2 import Environment, FileSystemLoader
+env = Environment(loader=FileSystemLoader(os.path.join(os.path.dirname(__file__), 'templates')))
 
 template_variables = {'pages': ['status', 'workers', 'tasks', 'system', 'profile', 'graph']}
 
+BOKEH_THEME = Theme(os.path.join(os.path.dirname(__file__), 'theme.yaml'))
 
 nan = float('nan')
 
@@ -181,6 +179,7 @@ class ProcessingHistogram(DashboardComponent):
 
             self.root = figure(title='Tasks Processing',
                                id='bk-nprocessing-histogram-plot',
+                               name='processing_hist',
                                **kwargs)
 
             self.root.xaxis.minor_tick_line_alpha = 0
@@ -213,6 +212,7 @@ class NBytesHistogram(DashboardComponent):
                                             'top': [0, 0]})
 
             self.root = figure(title='Bytes Stored',
+                               name='nbytes_hist',
                                id='bk-nbytes-histogram-plot',
                                **kwargs)
             self.root.xaxis[0].formatter = NumeralTickFormatter(format='0.0 b')
@@ -256,7 +256,7 @@ class CurrentLoad(DashboardComponent):
                                             'bokeh_address': ['', '']})
 
             processing = figure(title='Tasks Processing', tools='', id='bk-nprocessing-plot',
-                                width=int(width / 2), **kwargs)
+                                name='processing_hist', width=int(width / 2), **kwargs)
             rect = processing.rect(source=self.source,
                                    x='nprocessing-half', y='y',
                                    width='nprocessing', height=1,
@@ -266,7 +266,7 @@ class CurrentLoad(DashboardComponent):
 
             nbytes = figure(title='Bytes stored', tools='',
                             id='bk-nbytes-worker-plot', width=int(width / 2),
-                            **kwargs)
+                            name='nbytes_hist',  **kwargs)
             rect = nbytes.rect(source=self.source,
                                x='nbytes-half', y='y',
                                width='nbytes', height=1,
@@ -304,7 +304,6 @@ class CurrentLoad(DashboardComponent):
             self.nbytes_figure = nbytes
 
             processing.y_range = nbytes.y_range
-            self.root = row(nbytes, processing, sizing_mode='scale_width')
 
     def update(self):
         with log_errors():
@@ -326,13 +325,13 @@ class CurrentLoad(DashboardComponent):
                 else:
                     processing_color.append('blue')
 
-            nbytes = [ws.info['memory'] for ws in workers]
+            nbytes = [ws.metrics['memory'] for ws in workers]
             nbytes_text = [format_bytes(nb) for nb in nbytes]
             nbytes_color = []
             max_limit = 0
             for ws, nb in zip(workers, nbytes):
                 try:
-                    limit = self.scheduler.worker_info[ws.address]['memory_limit']
+                    limit = self.scheduler.workers[ws.address].memory_limit
                 except KeyError:
                     limit = 16e9
                 if limit > max_limit:
@@ -559,7 +558,7 @@ class TaskStream(components.TaskStream):
         if self.index == self.plugin.index:
             return
         with log_errors():
-            if self.index and self.source.data['start']:
+            if self.index and len(self.source.data['start']):
                 start = min(self.source.data['start'])
                 duration = max(self.source.data['duration'])
                 boundary = (self.offset + start - duration) / 1000
@@ -761,7 +760,7 @@ class TaskProgress(DashboardComponent):
         y_range = Range1d(-8, 0)
 
         self.root = figure(
-            id='bk-task-progress-plot', title='Progress',
+            id='bk-task-progress-plot', title='Progress', name='task_progress',
             x_range=x_range, y_range=y_range, toolbar_location=None, **kwargs
         )
         self.root.line(  # just to define early ranges
@@ -925,12 +924,17 @@ class WorkerTable(DashboardComponent):
     This is two plots, a text-based table for each host and a thin horizontal
     plot laying out hosts by their current memory use.
     """
+    excluded_names = {'executing', 'in_flight', 'in_memory', 'ready', 'time'}
 
     def __init__(self, scheduler, width=800, **kwargs):
         self.scheduler = scheduler
         self.names = ['worker', 'ncores', 'cpu', 'memory', 'memory_limit',
                       'memory_percent', 'num_fds', 'read_bytes', 'write_bytes',
                       'cpu_fraction']
+        workers = self.scheduler.workers.values()
+        self.extra_names = sorted({m for ws in workers
+                                   for m in ws.metrics
+                                   if m not in self.names} - self.excluded_names)
 
         table_names = ['worker', 'ncores', 'cpu', 'memory', 'memory_limit',
                        'memory_percent', 'num_fds', 'read_bytes',
@@ -951,20 +955,36 @@ class WorkerTable(DashboardComponent):
                       'num_fds': NumberFormatter(format='0'),
                       'ncores': NumberFormatter(format='0')}
 
+        if LooseVersion(bokeh.__version__) < '0.12.15':
+            dt_kwargs = {'row_headers': False}
+        else:
+            dt_kwargs = {'index_position': None}
+
         table = DataTable(
             source=self.source, columns=[columns[n] for n in table_names],
-            row_headers=False, reorderable=True, sortable=True, width=width,
+            reorderable=True, sortable=True, width=width, **dt_kwargs
         )
 
         for name in table_names:
             if name in formatters:
                 table.columns[table_names.index(name)].formatter = formatters[name]
 
+        extra_names = ['worker'] + self.extra_names
+        extra_columns = {name: TableColumn(field=name,
+                                           title=name.replace('_percent', '%'))
+                         for name in extra_names}
+
+        extra_table = DataTable(
+            source=self.source,
+            columns=[extra_columns[n] for n in extra_names],
+            reorderable=True, sortable=True, width=width, **dt_kwargs
+        )
+
         hover = HoverTool(
             point_policy="follow_mouse",
             tooltips="""
                 <div>
-                  <span style="font-size: 10px; font-family: Monaco, monospace;">@host: </span>
+                  <span style="font-size: 10px; font-family: Monaco, monospace;">@worker: </span>
                   <span style="font-size: 10px; font-family: Monaco, monospace;">@memory_percent</span>
                 </div>
                 """
@@ -1008,20 +1028,25 @@ class WorkerTable(DashboardComponent):
         else:
             sizing_mode = {}
 
-        self.root = column(cpu_plot, mem_plot, table, id='bk-worker-table', **sizing_mode)
+        components = [cpu_plot, mem_plot, table]
+        if self.extra_names:
+            components.append(extra_table)
+
+        self.root = column(*components, id='bk-worker-table', **sizing_mode)
 
     def update(self):
-        data = {name: [] for name in self.names}
-        for worker, info in sorted(self.scheduler.worker_info.items()):
-            for name in self.names:
-                data[name].append(info.get(name, None))
-            data['worker'][-1] = worker
-            if info['memory_limit']:
-                data['memory_percent'][-1] = info['memory'] / info['memory_limit']
+        data = {name: [] for name in self.names + self.extra_names}
+        for addr, ws in sorted(self.scheduler.workers.items()):
+            for name in self.names + self.extra_names:
+                data[name].append(ws.metrics.get(name, None))
+            data['worker'][-1] = ws.address
+            if ws.memory_limit:
+                data['memory_percent'][-1] = ws.metrics['memory'] / ws.memory_limit
             else:
                 data['memory_percent'][-1] = ''
-            data['cpu'][-1] = info['cpu'] / 100.0
-            data['cpu_fraction'][-1] = info['cpu'] / 100.0 / info['ncores']
+            data['memory_limit'][-1] = ws.memory_limit
+            data['cpu'][-1] = ws.metrics['cpu'] / 100.0
+            data['cpu_fraction'][-1] = ws.metrics['cpu'] / 100.0 / ws.ncores
 
         self.source.data.update(data)
 
@@ -1029,13 +1054,13 @@ class WorkerTable(DashboardComponent):
 def systemmonitor_doc(scheduler, extra, doc):
     with log_errors():
         sysmon = SystemMonitor(scheduler, sizing_mode='scale_width')
-        doc.title = "Dask Scheduler Internal Monitor"
+        doc.title = "Dask: Scheduler System Monitor"
         doc.add_periodic_callback(sysmon.update, 500)
 
         doc.add_root(column(sysmon.root, sizing_mode='scale_width'))
-        doc.template = template
-        doc.template_variables['active_page'] = 'system'
+        doc.template = env.get_template('simple.html')
         doc.template_variables.update(extra)
+        doc.theme = BOKEH_THEME
 
 
 def stealing_doc(scheduler, extra, doc):
@@ -1044,7 +1069,7 @@ def stealing_doc(scheduler, extra, doc):
         stealing_ts = StealingTimeSeries(scheduler, sizing_mode='scale_width')
         stealing_events = StealingEvents(scheduler, sizing_mode='scale_width')
         stealing_events.root.x_range = stealing_ts.root.x_range
-        doc.title = "Dask Workers Monitor"
+        doc.title = "Dask: Work Stealing"
         doc.add_periodic_callback(occupancy.update, 500)
         doc.add_periodic_callback(stealing_ts.update, 500)
         doc.add_periodic_callback(stealing_events.update, 500)
@@ -1053,9 +1078,9 @@ def stealing_doc(scheduler, extra, doc):
                             stealing_events.root,
                             sizing_mode='scale_width'))
 
-        doc.template = template
-        doc.template_variables['active_page'] = 'stealing'
+        doc.template = env.get_template('simple.html')
         doc.template_variables.update(extra)
+        doc.theme = BOKEH_THEME
 
 
 def events_doc(scheduler, extra, doc):
@@ -1063,11 +1088,11 @@ def events_doc(scheduler, extra, doc):
         events = Events(scheduler, 'all', height=250)
         events.update()
         doc.add_periodic_callback(events.update, 500)
-        doc.title = "Dask Scheduler Events"
+        doc.title = "Dask: Scheduler Events"
         doc.add_root(column(events.root, sizing_mode='scale_width'))
-        doc.template = template
-        doc.template_variables['active_page'] = 'events'
+        doc.template = env.get_template('simple.html')
         doc.template_variables.update(extra)
+        doc.theme = BOKEH_THEME
 
 
 def workers_doc(scheduler, extra, doc):
@@ -1075,11 +1100,11 @@ def workers_doc(scheduler, extra, doc):
         table = WorkerTable(scheduler)
         table.update()
         doc.add_periodic_callback(table.update, 500)
-        doc.title = "Dask Workers"
+        doc.title = "Dask: Workers"
         doc.add_root(table.root)
-        doc.template = template
-        doc.template_variables['active_page'] = 'workers'
+        doc.template = env.get_template('simple.html')
         doc.template_variables.update(extra)
+        doc.theme = BOKEH_THEME
 
 
 def tasks_doc(scheduler, extra, doc):
@@ -1088,70 +1113,152 @@ def tasks_doc(scheduler, extra, doc):
                         sizing_mode='stretch_both')
         ts.update()
         doc.add_periodic_callback(ts.update, 5000)
-        doc.title = "Dask Task Stream"
+        doc.title = "Dask: Task Stream"
         doc.add_root(ts.root)
-        doc.template = template
-        doc.template_variables['active_page'] = 'tasks'
+        doc.template = env.get_template('simple.html')
         doc.template_variables.update(extra)
+        doc.theme = BOKEH_THEME
 
 
 def graph_doc(scheduler, extra, doc):
     with log_errors():
         graph = GraphPlot(scheduler, sizing_mode='stretch_both')
+        doc.title = "Dask: Task Graph"
         graph.update()
         doc.add_periodic_callback(graph.update, 200)
         doc.add_root(graph.root)
 
-        doc.template = template
-        doc.template_variables['active_page'] = 'graph'
+        doc.template = env.get_template('simple.html')
         doc.template_variables.update(extra)
+        doc.theme = BOKEH_THEME
 
 
 def status_doc(scheduler, extra, doc):
     with log_errors():
         task_stream = TaskStream(scheduler, n_rectangles=1000,
-                                 clear_interval='10s', height=350)
+                                 clear_interval='10s', sizing_mode='stretch_both')
         task_stream.update()
         doc.add_periodic_callback(task_stream.update, 100)
 
-        task_progress = TaskProgress(scheduler, height=160)
+        task_progress = TaskProgress(scheduler, sizing_mode='stretch_both')
         task_progress.update()
         doc.add_periodic_callback(task_progress.update, 100)
 
         if len(scheduler.workers) < 50:
-            current_load = CurrentLoad(scheduler, height=160)
+            current_load = CurrentLoad(scheduler, sizing_mode='stretch_both')
             current_load.update()
             doc.add_periodic_callback(current_load.update, 100)
-            current_load_fig = current_load.root
+            doc.add_root(current_load.nbytes_figure)
+            doc.add_root(current_load.processing_figure)
         else:
-            nbytes_hist = NBytesHistogram(scheduler, width=300, height=160)
+            nbytes_hist = NBytesHistogram(scheduler, sizing_mode='stretch_both')
             nbytes_hist.update()
-            processing_hist = ProcessingHistogram(scheduler, width=300,
-                                                  height=160)
+            processing_hist = ProcessingHistogram(scheduler, sizing_mode='stretch_both')
             processing_hist.update()
             doc.add_periodic_callback(nbytes_hist.update, 100)
             doc.add_periodic_callback(processing_hist.update, 100)
             current_load_fig = row(nbytes_hist.root, processing_hist.root,
-                                   sizing_mode='scale_width')
+                                   sizing_mode='stretch_both')
 
-        doc.title = "Dask Status"
-        doc.add_root(column(current_load_fig,
-                            task_stream.root,
-                            task_progress.root,
-                            sizing_mode='scale_width'))
-        doc.template = template
-        doc.template_variables['active_page'] = 'status'
+            doc.add_root(nbytes_hist.root)
+            doc.add_root(processing_hist.root)
+
+        doc.title = "Dask: Status"
+        doc.add_root(task_progress.root)
+        doc.add_root(task_stream.root)
+        doc.theme = BOKEH_THEME
+        doc.template = env.get_template('status.html')
         doc.template_variables.update(extra)
+        doc.theme = BOKEH_THEME
+
+
+def individual_task_stream_doc(scheduler, extra, doc):
+    task_stream = TaskStream(scheduler, n_rectangles=1000,
+                             clear_interval='10s', sizing_mode='stretch_both')
+    task_stream.update()
+    doc.add_periodic_callback(task_stream.update, 100)
+    doc.add_root(task_stream.root)
+    doc.theme = BOKEH_THEME
+
+
+def individual_nbytes_doc(scheduler, extra, doc):
+    current_load = CurrentLoad(scheduler, sizing_mode='stretch_both')
+    current_load.update()
+    doc.add_periodic_callback(current_load.update, 100)
+    doc.add_root(current_load.nbytes_figure)
+    doc.theme = BOKEH_THEME
+
+
+def individual_nprocessing_doc(scheduler, extra, doc):
+    current_load = CurrentLoad(scheduler, sizing_mode='stretch_both')
+    current_load.update()
+    doc.add_periodic_callback(current_load.update, 100)
+    doc.add_root(current_load.processing_figure)
+    doc.theme = BOKEH_THEME
+
+
+def individual_progress_doc(scheduler, extra, doc):
+    task_progress = TaskProgress(scheduler, height=160, sizing_mode='stretch_both')
+    task_progress.update()
+    doc.add_periodic_callback(task_progress.update, 100)
+    doc.add_root(task_progress.root)
+    doc.theme = BOKEH_THEME
+
+
+def individual_graph_doc(scheduler, extra, doc):
+    with log_errors():
+        graph = GraphPlot(scheduler, sizing_mode='stretch_both')
+        graph.update()
+        doc.add_periodic_callback(graph.update, 200)
+        doc.add_root(graph.root)
+        doc.theme = BOKEH_THEME
+
+
+def individual_profile_doc(scheduler, extra, doc):
+    with log_errors():
+        prof = ProfileTimePlot(scheduler, sizing_mode='scale_width', doc=doc)
+        doc.add_root(prof.root)
+        prof.trigger_update()
+        doc.theme = BOKEH_THEME
+
+
+def individual_profile_server_doc(scheduler, extra, doc):
+    with log_errors():
+        prof = ProfileServer(scheduler, sizing_mode='scale_width', doc=doc)
+        doc.add_root(prof.root)
+        prof.trigger_update()
+        doc.theme = BOKEH_THEME
+
+
+def individual_workers_doc(scheduler, extra, doc):
+    with log_errors():
+        table = WorkerTable(scheduler)
+        table.update()
+        doc.add_periodic_callback(table.update, 500)
+        doc.add_root(table.root)
+        doc.theme = BOKEH_THEME
 
 
 def profile_doc(scheduler, extra, doc):
     with log_errors():
-        doc.title = "Dask Profile"
+        doc.title = "Dask: Profile"
         prof = ProfileTimePlot(scheduler, sizing_mode='scale_width', doc=doc)
         doc.add_root(prof.root)
-        doc.template = template
-        doc.template_variables['active_page'] = 'profile'
+        doc.template = env.get_template('simple.html')
         doc.template_variables.update(extra)
+        doc.theme = BOKEH_THEME
+
+        prof.trigger_update()
+
+
+def profile_server_doc(scheduler, extra, doc):
+    with log_errors():
+        doc.title = "Dask: Profile of Event Loop"
+        prof = ProfileServer(scheduler, sizing_mode='scale_width', doc=doc)
+        doc.add_root(prof.root)
+        doc.template = env.get_template('simple.html')
+        doc.template_variables.update(extra)
+        doc.theme = BOKEH_THEME
 
         prof.trigger_update()
 
@@ -1168,28 +1275,31 @@ class BokehScheduler(BokehServer):
         self.server_kwargs = kwargs
         self.server_kwargs['prefix'] = prefix or None
 
-        systemmonitor = Application(FunctionHandler(partial(systemmonitor_doc, scheduler, self.extra)))
-        workers = Application(FunctionHandler(partial(workers_doc, scheduler, self.extra)))
-        stealing = Application(FunctionHandler(partial(stealing_doc, scheduler, self.extra)))
-        counters = Application(FunctionHandler(partial(counters_doc, scheduler, self.extra)))
-        events = Application(FunctionHandler(partial(events_doc, scheduler, self.extra)))
-        tasks = Application(FunctionHandler(partial(tasks_doc, scheduler, self.extra)))
-        status = Application(FunctionHandler(partial(status_doc, scheduler, self.extra)))
-        profile = Application(FunctionHandler(partial(profile_doc, scheduler, self.extra)))
-        graph = Application(FunctionHandler(partial(graph_doc, scheduler, self.extra)))
-
         self.apps = {
-            '/system': systemmonitor,
-            '/stealing': stealing,
-            '/workers': workers,
-            '/events': events,
-            '/counters': counters,
-            '/tasks': tasks,
-            '/status': status,
-            '/profile': profile,
-            '/graph': graph,
+            '/system': systemmonitor_doc,
+            '/stealing': stealing_doc,
+            '/workers': workers_doc,
+            '/events': events_doc,
+            '/counters': counters_doc,
+            '/tasks': tasks_doc,
+            '/status': status_doc,
+            '/profile': profile_doc,
+            '/profile-server': profile_server_doc,
+            '/graph': graph_doc,
+
+            '/individual-task-stream': individual_task_stream_doc,
+            '/individual-progress': individual_progress_doc,
+            '/individual-graph': individual_graph_doc,
+            '/individual-profile': individual_profile_doc,
+            '/individual-profile-server': individual_profile_server_doc,
+            '/individual-nbytes': individual_nbytes_doc,
+            '/individual-nprocessing': individual_nprocessing_doc,
+            '/individual-workers': individual_workers_doc,
         }
 
+        self.apps = {k: partial(v, scheduler, self.extra)
+                     for k, v in self.apps.items()}
+
         self.loop = io_loop or scheduler.loop
         self.server = None
 
@@ -1207,4 +1317,5 @@ class BokehScheduler(BokehServer):
         from .scheduler_html import routes
         handlers = [(self.prefix + '/' + url, cls, {'server': self.my_server, 'extra': self.extra})
                     for url, cls in routes]
+
         self.server._tornado.add_handlers(r'.*', handlers)
