@@ -13,9 +13,10 @@ from bokeh.models import (ColumnDataSource, DataRange1d, HoverTool,
 from bokeh.models.widgets import DataTable, TableColumn
 from bokeh.plotting import figure
 from bokeh.palettes import RdBu
+from bokeh.themes import Theme
 from toolz import merge, partition_all
 
-from .components import DashboardComponent, ProfileTimePlot
+from .components import DashboardComponent, ProfileTimePlot, ProfileServer
 from .core import BokehServer
 from .utils import transpose
 from ..compatibility import WINDOWS
@@ -26,12 +27,14 @@ from ..utils import (log_errors, key_split, format_bytes, format_time)
 
 logger = logging.getLogger(__name__)
 
-import jinja2
-
-with open(os.path.join(os.path.dirname(__file__), 'template.html')) as f:
+with open(os.path.join(os.path.dirname(__file__), 'templates', 'base.html')) as f:
     template_source = f.read()
 
-template = jinja2.Template(template_source)
+from jinja2 import Environment, FileSystemLoader
+env = Environment(loader=FileSystemLoader(os.path.join(os.path.dirname(__file__), 'templates')))
+
+BOKEH_THEME = Theme(os.path.join(os.path.dirname(__file__), 'theme.yaml'))
+
 template_variables = {'pages': ['main', 'system', 'profile', 'crossfilter']}
 
 
@@ -159,7 +162,7 @@ class CommunicatingTimeSeries(DashboardComponent):
 
         fig = figure(title="Communication History",
                      x_axis_type='datetime',
-                     y_range=[-0.1, worker.total_connections + 0.5],
+                     y_range=[-0.1, worker.total_out_connections + 0.5],
                      height=150, tools='', x_range=x_range, **kwargs)
         fig.line(source=self.source, x='x', y='in', color='red')
         fig.line(source=self.source, x='x', y='out', color='blue')
@@ -560,9 +563,10 @@ def main_doc(worker, extra, doc):
                             communicating_ts.root,
                             communicating_stream.root,
                             sizing_mode='scale_width'))
-        doc.template = template
+        doc.template = env.get_template('simple.html')
         doc.template_variables['active_page'] = 'main'
         doc.template_variables.update(extra)
+        doc.theme = BOKEH_THEME
 
 
 def crossfilter_doc(worker, extra, doc):
@@ -575,9 +579,10 @@ def crossfilter_doc(worker, extra, doc):
         doc.add_periodic_callback(crossfilter.update, 500)
 
         doc.add_root(column(statetable.root, crossfilter.root))
-        doc.template = template
+        doc.template = env.get_template('simple.html')
         doc.template_variables['active_page'] = 'crossfilter'
         doc.template_variables.update(extra)
+        doc.theme = BOKEH_THEME
 
 
 def systemmonitor_doc(worker, extra, doc):
@@ -587,9 +592,10 @@ def systemmonitor_doc(worker, extra, doc):
         doc.add_periodic_callback(sysmon.update, 500)
 
         doc.add_root(sysmon.root)
-        doc.template = template
+        doc.template = env.get_template('simple.html')
         doc.template_variables['active_page'] = 'system'
         doc.template_variables.update(extra)
+        doc.theme = BOKEH_THEME
 
 
 def counters_doc(server, extra, doc):
@@ -599,9 +605,10 @@ def counters_doc(server, extra, doc):
         doc.add_periodic_callback(counter.update, 500)
 
         doc.add_root(counter.root)
-        doc.template = template
+        doc.template = env.get_template('simple.html')
         doc.template_variables['active_page'] = 'counters'
         doc.template_variables.update(extra)
+        doc.theme = BOKEH_THEME
 
 
 def profile_doc(server, extra, doc):
@@ -611,9 +618,23 @@ def profile_doc(server, extra, doc):
         profile.trigger_update()
 
         doc.add_root(profile.root)
-        doc.template = template
+        doc.template = env.get_template('simple.html')
         doc.template_variables['active_page'] = 'profile'
         doc.template_variables.update(extra)
+        doc.theme = BOKEH_THEME
+
+
+def profile_server_doc(server, extra, doc):
+    with log_errors():
+        doc.title = "Dask: Profile of Event Loop"
+        prof = ProfileServer(server, sizing_mode='scale_width', doc=doc)
+        doc.add_root(prof.root)
+        doc.template = env.get_template('simple.html')
+        # doc.template_variables['active_page'] = ''
+        doc.template_variables.update(extra)
+        doc.theme = BOKEH_THEME
+
+        prof.trigger_update()
 
 
 class BokehWorker(BokehServer):
@@ -636,12 +657,14 @@ class BokehWorker(BokehServer):
         systemmonitor = Application(FunctionHandler(partial(systemmonitor_doc, worker, extra)))
         counters = Application(FunctionHandler(partial(counters_doc, worker, extra)))
         profile = Application(FunctionHandler(partial(profile_doc, worker, extra)))
+        profile_server = Application(FunctionHandler(partial(profile_server_doc, worker, extra)))
 
         self.apps = {'/main': main,
                      '/counters': counters,
                      '/crossfilter': crossfilter,
                      '/system': systemmonitor,
-                     '/profile': profile}
+                     '/profile': profile,
+                     '/profile-server': profile_server}
 
         self.loop = io_loop or worker.loop
         self.server = None
